@@ -125,6 +125,10 @@ def main():
     help='Run the command including the --terragrunt-source-update option',
     show_default=True
     )
+@click.option('--planfile', '-p', is_flag=True, default=False,
+    help='Use the generated planfile when applying the changes',
+    show_default=True
+    )
 @click.option('--auto-approve', '-a', is_flag=True, default=False,
     help='Do not ask for confirmation before applying planned changes (where applicable)',
     show_default=True
@@ -136,7 +140,7 @@ def main():
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
 @click.version_option(version=__version__)
 def run(command, verbose, debug, dry_run, no_lock, update_source,
-    auto_approve, working_dir, terragrunt_args):
+    planfile, auto_approve, working_dir, terragrunt_args):
     """ [default] Executes a terragrunt command on a single project """
 
     check_latest_version()
@@ -148,6 +152,7 @@ def run(command, verbose, debug, dry_run, no_lock, update_source,
         dry_run=dry_run,
         no_lock=no_lock,
         update_source=update_source,
+        planfile=planfile,
         auto_approve=auto_approve,
         working_dir=working_dir,
         terragrunt_args=terragrunt_args,
@@ -188,7 +193,15 @@ def run(command, verbose, debug, dry_run, no_lock, update_source,
     help='Run the command step by step and stop when an error occurs (where applicable)',
     show_default=True
     )
-@click.option('--auto-approve/--no-auto-approve', is_flag=True, default=True,
+@click.option('--planfile', '-p', is_flag=True, default=False,
+    help='Use the generated planfile when applying the changes',
+    show_default=True
+    )
+# @click.option('--auto-approve/--no-auto-approve', is_flag=True, default=True,
+#     help='Do not ask for confirmation before applying planned changes (where applicable)',
+#     show_default=True
+#     )
+@click.option('--auto-approve', '-a', is_flag=True, default=False,
     help='Do not ask for confirmation before applying planned changes (where applicable)',
     show_default=True
     )
@@ -203,7 +216,7 @@ def run(command, verbose, debug, dry_run, no_lock, update_source,
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
 @click.version_option(version=__version__)
 def run_all(command, verbose, debug, dry_run, no_lock, update_source, exclude_external_dependencies,
-    step_by_step, auto_approve, working_dir, limit_parallelism, terragrunt_args):
+    step_by_step, planfile, auto_approve, working_dir, limit_parallelism, terragrunt_args):
     """ Executes a terragrunt command across multiple projects """
 
     check_latest_version()
@@ -217,6 +230,7 @@ def run_all(command, verbose, debug, dry_run, no_lock, update_source, exclude_ex
         update_source=update_source,
         exclude_external_dependencies=exclude_external_dependencies,
         step_by_step=step_by_step,
+        planfile=planfile,
         auto_approve=auto_approve,
         working_dir=working_dir,
         limit_parallelism=limit_parallelism,
@@ -292,9 +306,13 @@ def run_import(address, id, verbose, dry_run, working_dir, no_lock, terragrunt_a
     '--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
+@click.option('--out', '-o', is_flag=True, default=False,
+    help='Show output as json',
+    show_default=True
+    )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
 @click.version_option(version=__version__)
-def analyze(verbose, dry_run, exclude_external_dependencies, working_dir, terragrunt_args):
+def analyze(verbose, dry_run, exclude_external_dependencies, working_dir, out, terragrunt_args):
     """ Analyzes the plan files """
 
     check_latest_version()
@@ -304,6 +322,7 @@ def analyze(verbose, dry_run, exclude_external_dependencies, working_dir, terrag
         dry_run=dry_run,
         exclude_external_dependencies=exclude_external_dependencies,
         working_dir=working_dir,
+        out=out,
         terragrunt_args=terragrunt_args,
     )
 
