@@ -24,69 +24,8 @@ import click
 import hcl
 import networkx as nx
 
-class Printer():
-    """ A simple class for printing nice messages """
-    def __init__(
-        self,
-        verbose: bool,
-        ):
-
-        self._print_verbose = verbose
-
-    @property
-    def print_verbose(self):
-        return self._print_verbose
-
-    def line(self):
-        width = 80
-        try:
-          width, _ = shutil.get_terminal_size() 
-        except OSError as e:
-          pass
-        click.secho("-" * width, file=sys.stderr)
-
-    def header(self, msg, print_line_before=False, print_line_after=False):
-        msg = msg.strip() if isinstance(msg, str) else msg
-        self.line() if print_line_before else None
-        click.secho('\n' + msg, bold=True, file=sys.stderr)
-        self.line() if print_line_after else None
-
-    def verbose(self, msg, print_line_before=False, print_line_after=False):
-        if self._print_verbose:
-            msg = msg.strip() if isinstance(msg, str) else msg
-            self.line() if print_line_before else None
-            print(msg, flush=True, file=sys.stderr)
-            self.line() if print_line_after else None
-
-    def normal(self, msg, print_line_before=False, print_line_after=False):
-        msg = msg.strip() if isinstance(msg, str) else msg
-        self.line() if print_line_before else None
-        print(msg, flush=True, file=sys.stderr)
-        self.line() if print_line_after else None
-
-    def bold(self, msg, print_line_before=False, print_line_after=False):
-        msg = msg.strip() if isinstance(msg, str) else msg
-        self.line() if print_line_before else None
-        click.secho('\n' + msg, bold=True, file=sys.stderr)
-        self.line() if print_line_after else None
-
-    def warning(self, msg, print_line_before=False, print_line_after=False):
-        msg = msg.strip() if isinstance(msg, str) else msg
-        self.line() if print_line_before else None
-        click.secho(msg, fg="yellow", bold=True, file=sys.stderr)
-        self.line() if print_line_after else None
-
-    def error(self, msg, print_line_before=False, print_line_after=False):
-        msg = msg.strip() if isinstance(msg, str) else msg
-        self.line() if print_line_before else None
-        click.secho(msg, fg="red", bold=True, file=sys.stderr)
-        self.line() if print_line_after else None
-
-    def success(self, msg, print_line_before=False, print_line_after=False):
-        msg = msg.strip() if isinstance(msg, str) else msg
-        self.line() if print_line_before else None
-        click.secho(msg, fg="green", bold=True, file=sys.stderr)
-        self.line() if print_line_after else None
+from .printer import Printer
+from .terrasafe import run_terrasafe
 
 class TgWrap():
     """
@@ -115,15 +54,15 @@ class TgWrap():
 
     def _construct_command(self, command, debug, exclude_external_dependencies,
         non_interactive=True, no_auto_approve=True, no_lock=True, update_source=False,
-        working_dir=None, limit_parallelism=None, terragrunt_args=()):
+        planfile=None, working_dir=None, limit_parallelism=None, terragrunt_args=()):
         """ Constructs the command """
         commands = {
             'generic': '{base_command} {command} --terragrunt-non-interactive {no_auto_approve} {ignore_deps} {debug_level} {update_source} {working_dir} {parallelism} {terragrunt_args}',
             'info': '{base_command} terragrunt-info --terragrunt-non-interactive {ignore_deps} {debug_level} {update_source} {working_dir} {terragrunt_args}',
             'plan': '{base_command} {command} --terragrunt-non-interactive  -out=planfile {ignore_deps} {debug_level} {lock_level} {update_source} {working_dir} {parallelism} {terragrunt_args}',
-            'apply': '{base_command} {command} {non_interactive} {no_auto_approve} {parallelism} {ignore_deps} {debug_level} {update_source} {working_dir} {parallelism} {terragrunt_args}',
+            'apply': '{base_command} {command} {non_interactive} {no_auto_approve} {parallelism} {ignore_deps} {debug_level} {update_source} {working_dir} {parallelism} {terragrunt_args} {planfile}',
             'show': '{base_command} {command} --terragrunt-non-interactive {ignore_deps} {update_source} -json planfile', # no working dir allowed here!!!
-            'destroy': '{base_command} {command} {non_interactive} {no_auto_approve} {ignore_deps} {debug_level} {working_dir} {parallelism} {terragrunt_args}',
+            'destroy': '{base_command} {command} {non_interactive} {no_auto_approve} {ignore_deps} {debug_level} {working_dir} {parallelism} {terragrunt_args} {planfile}',
         }
 
         lock_stmt         = '-lock=false' if no_lock else ''
@@ -134,6 +73,7 @@ class TgWrap():
         interactive_stmt  = '--terragrunt-non-interactive' if non_interactive else ''
         working_dir_stmt  = f'--terragrunt-working-dir {working_dir}' if working_dir else ''
         parallelism_stmt  = f'--terragrunt-parallelism {limit_parallelism}' if limit_parallelism else ''
+        planfile_stmt     = 'planfile' if planfile else ''
 
         base_command      = 'terragrunt run-all'
 
@@ -149,6 +89,7 @@ class TgWrap():
                 non_interactive=interactive_stmt,
                 working_dir=working_dir_stmt,
                 parallelism=parallelism_stmt,
+                planfile=planfile_stmt,
                 terragrunt_args=' '.join(terragrunt_args),
             )
         else:
@@ -234,18 +175,18 @@ class TgWrap():
 
         return error
 
-    def _run_graph_step(self, command, working_dir, directory, collect_output_file, dry_run):
+    def _run_graph_step(self, command, working_dir, module, collect_output_file, dry_run, progress):
         """ Runs a step in the graph """
 
         stop_processing = False
 
         # if we have a specific working dir, and the dir is relative, combine the two
-        if working_dir and not os.path.isabs(directory):
-            directory = os.path.join(os.path.abspath(working_dir), directory)
+        if working_dir and not os.path.isabs(module):
+            module = os.path.join(os.path.abspath(working_dir), module)
             self.printer.verbose(f'Executing in directory: {working_dir}')
 
         self.printer.header(
-            f'\n\nStart processing directory: {directory}\n\n',
+            f'\n\nStart processing module: {module} ({progress})\n\n',
             print_line_before=True,
             )
 
@@ -256,7 +197,7 @@ class TgWrap():
                     )
             else:
                 if collect_output_file:
-                    collect_output_file.write(f'{directory}{self.SEPARATOR}')
+                    collect_output_file.write(f'{module}{self.SEPARATOR}')
                     collect_output_file.flush()
 
                 messages = ""
@@ -267,7 +208,7 @@ class TgWrap():
                     rc = subprocess.run(
                         shlex.split(command),
                         text=True,
-                        cwd=directory,
+                        cwd=module,
                         stdout=collect_output_file if collect_output_file else sys.stdout,
                         stderr=f,
                     )
@@ -283,20 +224,20 @@ class TgWrap():
 
                 if error:
                     raise Exception(
-                        f'An error situation detected while processing the terragrunt dependencies graph in directory {directory}'
+                        f'An error situation detected while processing the terragrunt dependencies graph in directory {module}'
                         )
                 else:
                     self.printer.success(
-                        f'Directory {directory} processed successfully',
+                        f'Directory {module} processed successfully',
                     )
 
         except FileNotFoundError:
-            self.printer.warning(f'Directory {directory} not found, continue')
+            self.printer.warning(f'Directory {module} not found, continue')
         except Exception as e:
             self.printer.error(f"Error occurred:\n{str(e)}")
             self.printer.error("Full stack:", print_line_before=True)
             self.printer.normal(messages, print_line_after=True)
-            self.printer.normal(f"Directory {directory} failed!")
+            self.printer.normal(f"Directory {module} failed!")
 
             stop_processing = True
         finally:
@@ -326,8 +267,8 @@ class TgWrap():
             self.printer.header("The following groups will be processed:")
             for idx, group in enumerate(groups):
                 self.printer.normal(f"\nGroup {idx+1}:")
-                for directory in group:
-                    self.printer.normal(f"- {directory}")
+                for module in group:
+                    self.printer.normal(f"- {module}")
 
         if ask_for_confirmation:
             response = input("\nDo you want to continue? (y/n) ")
@@ -335,18 +276,23 @@ class TgWrap():
                 sys.exit(1)
 
         stop_processing = False
+        nbr_of_groups = len(groups)
         for idx, group in enumerate(groups):
+            
             self.printer.header(f'Group {idx+1}')
             self.printer.normal(group)
 
             if command:
-                for directory in group:
+                nbr_of_modules = len(group)
+                for idx2, module in enumerate(group):
+                    progress = f'module #{idx2+1}/{nbr_of_modules} of group #{idx+1}/{nbr_of_groups}'
                     stop_processing = self._run_graph_step(
                         command=command,
                         working_dir=working_dir,
-                        directory=directory,
+                        module=module,
                         collect_output_file=collect_output_file,
                         dry_run=dry_run,
+                        progress=progress,
                     )
 
                     if stop_processing:
@@ -412,7 +358,7 @@ class TgWrap():
             self.printer.verbose(rc)
 
     def run(self, command, debug, dry_run, no_lock, update_source,
-        auto_approve, working_dir, terragrunt_args):
+        planfile, auto_approve, working_dir, terragrunt_args):
         """ Executes a terragrunt command on a single module """
 
         self.printer.verbose(f"Attempting to execute 'run {command}'")
@@ -434,6 +380,7 @@ class TgWrap():
             exclude_external_dependencies=True,
             no_lock=no_lock,
             update_source=update_source,
+            planfile=planfile,
             no_auto_approve=(not auto_approve),
             working_dir=working_dir,
             terragrunt_args=terragrunt_args,
@@ -449,7 +396,7 @@ class TgWrap():
             self.printer.verbose(rc)
 
     def run_all(self, command, debug, dry_run, no_lock, update_source,
-        exclude_external_dependencies, step_by_step, auto_approve, 
+        exclude_external_dependencies, step_by_step, planfile, auto_approve, 
         working_dir, limit_parallelism, terragrunt_args):
         """ Executes a terragrunt command across multiple modules """
 
@@ -457,14 +404,19 @@ class TgWrap():
         if terragrunt_args:
             self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
 
+        # auto approve is only relevant with a modifying command
+        modifying_command = (command.lower() in ['apply', 'destroy'])
+        auto_approve = auto_approve if modifying_command else True
+
         cmd = self._construct_command(
             command=command,
             debug=debug,
             exclude_external_dependencies=True if step_by_step else exclude_external_dependencies,
-            non_interactive=auto_approve,
+            non_interactive=True if step_by_step else auto_approve,
             no_lock=no_lock,
             update_source=update_source,
-            no_auto_approve=(not auto_approve),
+            planfile=planfile,
+            no_auto_approve=False if step_by_step else (not auto_approve),
             working_dir=None if step_by_step else working_dir,
             terragrunt_args=terragrunt_args,
             limit_parallelism=limit_parallelism,
@@ -516,11 +468,14 @@ class TgWrap():
         else:
             env = os.environ.copy()
             # TERRAGRUNT_SOURCE should not be present (or it should be a fully qualified path (which is typically not the case))
-            value = env.pop('TERRAGRUNT_SOURCE')
-            if value:
-                self.printer.verbose(
-                    f'Terragrunt source environment variable with value {value} will be ignored'
-                    )
+            try:
+                value = env.pop('TERRAGRUNT_SOURCE')
+                if value:
+                    self.printer.verbose(
+                        f'Terragrunt source environment variable with value {value} will be ignored'
+                        )
+            except KeyError:
+                pass
 
             # the `posix=False` is to prevent the split command to remove quotes from strings,
             # e.g. when executing commands like this:
@@ -531,7 +486,7 @@ class TgWrap():
             )
             self.printer.verbose(rc)
 
-    def analyze(self, dry_run, exclude_external_dependencies, working_dir, terragrunt_args):
+    def analyze(self, dry_run, exclude_external_dependencies, working_dir, out, terragrunt_args):
         """ Analyzes the plan files """
 
         self.printer.verbose("Attempting to 'analyze'")
@@ -546,7 +501,29 @@ class TgWrap():
             terragrunt_args=terragrunt_args,
             )
 
+        # Check if the "TERRASAFE_CONFIG" environment variable is set
+        env_var = "TERRASAFE_CONFIG"
+        ts_config = None
+        if not env_var in os.environ:
+            self.printer.warning(
+                f"{env_var} environment variable is not set, this is required for running the terrasafe command!"
+                )
+        # and if it exists as a file
+        elif not os.path.exists(os.environ.get(env_var)):
+            self.printer.warning(
+                f"{env_var} environment variable is set, but the file it points to does not exist!"
+                )
+        else:
+            config_path = os.environ.get(env_var)
+            self.printer.verbose(
+                f"\nRun terrasafe using config {config_path}"
+                )
+
+            config_file = open(config_path)
+            ts_config = json.load(config_file)
+
         ts_validation_successful = True
+        changes = {}
         try:
             # then run it and capture the output
             with tempfile.NamedTemporaryFile(mode='w+', prefix='tgwrap-', delete=False) as f:
@@ -567,74 +544,66 @@ class TgWrap():
                     module = split_line[0]
                     plan_file = split_line[1]
 
-                    self.printer.header(f"Analyse module: {module}")
+                    changes[module] = {
+                        "deletions" : [],
+                        "other" : [],
+                        "unauthorized" : [],
+                    }
 
                     try:
-                        d = json.loads(plan_file)
+                        data = json.loads(plan_file)
 
-                        if 'resource_changes' in d and len(d['resource_changes']) > 0:
-                            self.printer.header('Changes:')
-
-                            changes = False
-                            for rc in d['resource_changes']:
+                        if 'resource_changes' in data and len(data['resource_changes']) > 0:
+                            for rc in data['resource_changes']:
                                 # check if we do have actual changes
                                 actions = rc['change']['actions']
+                                action = f'{rc["address"]}: {", ".join(actions)}'
                                 if len(actions) == 1 and (actions[0] == 'no-op' or actions[0] == 'read'):
                                     pass # ignore, just an state change
                                 elif 'delete' in actions:
-                                    self.printer.warning(f'- {rc["address"]}: {", ".join(actions)}')
-                                    changes = True
+                                    changes[module]["deletions"].append(action)
                                 else:
-                                    self.printer.normal(f'- {rc["address"]}: {", ".join(actions)}')
-                                    changes = True
+                                    changes[module]["other"].append(action)
 
-                            if not changes:
-                                print('- no real changes detected.')
-
-                        # if so, write to file
-                        json_file = tempfile.NamedTemporaryFile(mode='w+', prefix='tgwrap-', delete=True)
-                        self.printer.verbose(f"Opened temp file for terrasafe input: {json_file.name}")
-
-                        json_file.write(plan_file)
-                        json_file.flush()
-
-                        # Check if the "TERRASAFE_CONFIG" environment variable is set
-                        env_var = "TERRASAFE_CONFIG"
-                        if not env_var in os.environ:
-                            self.printer.warning(
-                                f"{env_var} environment variable is not set, this is required for running the terrasafe command!"
+                        if ts_config:
+                            changes[module]["unauhhorized"] = run_terrasafe(
+                                config=ts_config,
+                                data=data,
+                                verbose=self.printer.print_verbose,
                                 )
-                        else:
-                            self.printer.header(
-                                f"\nRun terrasafe using config {os.environ.get(env_var)}"
-                                )
-
-                            cmd = f"cat {json_file.name} | terrasafe --config {os.environ.get('TERRASAFE_CONFIG')}"
-                            output = subprocess.run(
-                                cmd,
-                                shell=True,
-                                text=True,
-                                capture_output=True,
-                                )
-                            if output.returncode != 0:
+                            if len(changes[module]["unauhhorized"]) > 0:
                                 ts_validation_successful = False
-                                self.printer.error(output)
-                            elif '0 unauthorized' in output.stdout:
-                                self.printer.success(output.stdout)
                     except json.decoder.JSONDecodeError as e:
                         raise Exception(
                             f"Planfile for {module} was no proper json, further analysis not possible."
                             ) from e
-                    finally:
-                        try:
-                            json_file.close()
-                        except UnboundLocalError:
-                            pass
         finally:
             os.remove(f.name)
 
+        self.printer.header("Analysis results:", print_line_before=True)
+        for key, value in changes.items():
+            self.printer.header(f'Module: {key}')
+            if not (value["deletions"] or value["other"] or value["unauthorized"]):
+                self.printer.success('- No changes detected')
+            if value["unauthorized"]:
+                self.printer.error('- Unauthorized deletions:')
+                for m in value["unauhtorized"]:
+                    self.printer.error(f' -> {m}')
+            if value["deletions"]:
+                self.printer.warning('- Deletions:')
+                for m in value["deletions"]:
+                    self.printer.warning(f' -> {m}')
+            if value["other"]:
+                self.printer.normal('- Other:')
+                for m in value["other"]:
+                    self.printer.normal(f' -> {m}')
+
+        if out:
+            print(json.dumps(changes, indent=4))
+
         if not ts_validation_successful:
-            raise Exception("Terrasafe validation failed on one or more modules")
+            self.printer.error("Terrasafe validation failed on one or more modules")
+            sys.exit(1)
 
     def deploy(
         self, source_stage, target_stage, source_domain, target_domain, module,
